/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "glView.h"

GLView::GLView( QWidget *parent, const char *name)
	: QGLWidget( parent, name)
{
	init();
}

GLView::GLView( QWidget *parent, const char *name,
	const QGLWidget *p_share)
	: QGLWidget( parent, name, p_share)
{
	init();
}

GLView::~GLView()
{
	if( on_destructor) on_destructor( this);
	emit sig_destroy( this);
}

void GLView::init()
{
	edata = 0;
	on_initialize = 0;
	on_destructor = 0;
	on_paint = 0;
	on_resize = 0;
	on_mouse_down = 0;
	on_mouse_up = 0;
	on_mouse_move = 0;
	on_key_down = 0;
}

void GLView::paintGL()
{
	emit sig_paintGL( this);
	if( on_paint) {
		makeCurrent();
		on_paint( this);
	}
}

void GLView::resizeGL( int w, int h)
{
	emit sig_resizeGL( w, h, this);
	if( on_resize) on_resize( w, h, this);
}

void GLView::initializeGL()
{
	if( on_initialize) {
		makeCurrent();
		on_initialize( this);
	}
	emit sig_initializeGL( this);
}

void GLView::mousePressEvent( QMouseEvent *e)
{
	emit sig_mouse_down( e, this);
	if( on_mouse_down) {
		makeCurrent();
		on_mouse_down( e, this);
	}
}

void GLView::mouseReleaseEvent( QMouseEvent *e)
{
	emit sig_mouse_up( e, this);
	if( on_mouse_up) {
		makeCurrent();
		on_mouse_up( e, this);
	}
}

void GLView::mouseMoveEvent( QMouseEvent *e)
{
	emit sig_mouse_move( e, this);
	if( on_mouse_move) {
		makeCurrent();
		on_mouse_move( e, this);
	}
}

void GLView::keyPressEvent( QKeyEvent *e)
{
	emit sig_key_down( e, this);
	if( on_key_down) {
		makeCurrent();
		on_key_down( e, this);
	}
}
