/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef GLBOX_H
#define GLBOX_H

#include <qgl.h>
#include <rgPort.h>
#include <rgOpenGL.h>

class GLView : public QGLWidget
{
	Q_OBJECT
public:

	rgGLViewMatrix vmat;

	void *edata;
	void (*on_destructor)( GLView *g);
	void (*on_paint)( GLView *g);
	void (*on_resize)( int w, int h, GLView *g);
	void (*on_mouse_down)( QMouseEvent *e, GLView *g);
	void (*on_mouse_up)( QMouseEvent *e, GLView *g);
	void (*on_mouse_move)( QMouseEvent *e, GLView *g);
	void (*on_key_down)( QKeyEvent *e, GLView *g);
	void (*on_initialize)( GLView *g);

	GLView( QWidget *parent, const char *name);
	GLView( QWidget *parent, const char *name, const QGLWidget *p_share);
	~GLView();

	rgVector view_normal() {
		return (vmat.icamR * rgVector( 0, 0, 1));};

	rgVector up_vector() {
		return (vmat.icamR * rgVector( 0, 1, 0));};

	rgVector side_vector() {
		return (vmat.icamR * rgVector( 1, 0, 0));};

	void paintGL();
	void resizeGL( int w, int h);
	void initializeGL();

	void mousePressEvent( QMouseEvent *e);
	void mouseReleaseEvent( QMouseEvent *e);
	void mouseMoveEvent( QMouseEvent *e);
	void keyPressEvent( QKeyEvent *e);
    
public slots:

	void init();

signals:
	void sig_paintGL( GLView *g);
	void sig_resizeGL( int w, int h, GLView *g);
	void sig_initializeGL( GLView *g);
	void sig_mouse_down( QMouseEvent *e, GLView *g);
	void sig_mouse_move( QMouseEvent *e, GLView *g);
	void sig_mouse_up( QMouseEvent *e, GLView *g);
	void sig_key_down( QKeyEvent *e, GLView *g);
	void sig_destroy( GLView *g);
};

#endif
