/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <qlistbox.h>

class QDragableListBox : public QListBox
{
	Q_OBJECT
public:
	QDragableListBox( QWidget *parent = 0, const char *name = 0, WFlags f = 0) :
		QListBox( parent, name, f) {
		m_drag = false;
		m_drag_on_left = true;
		m_drag_on_middle = false;
		m_drag_on_right = false;
	}

	bool dragOnLeft() const { return m_drag_on_left;}
	bool dragOnMiddle() const { return m_drag_on_middle;}
	bool dragOnRight() const { return m_drag_on_right;}

	void setDragOnLeft( const bool b) { m_drag_on_left = b;}
	void setDragOnMiddle( const bool b) { m_drag_on_middle = b;}
	void setDragOnRight( const bool b) { m_drag_on_right = b;}

public slots:
	void mousePressEvent( QMouseEvent *e) {
		QListBox::mousePressEvent( e);
		if( (Qt::LeftButton == e->button()) && dragOnLeft())
			m_drag = true;
		if( (Qt::MidButton == e->button()) && dragOnMiddle())
			m_drag = true;
		if( (Qt::RightButton == e->button()) && dragOnRight())
			m_drag = true;
	}

	void mouseMoveEvent( QMouseEvent *e) {
		if( m_drag) {
			m_drag = false;
			emit do_drag();
		} else QListBox::mouseMoveEvent( e);
	}

	void mouseReleaseEvent( QMouseEvent *e) {
		m_drag = false;
		QListBox::mouseReleaseEvent( e);
	}

	void dropEvent( QDropEvent *e) {
		emit on_drop( e);
	}

	void dragEnterEvent( QDragEnterEvent *e) {
		emit on_drag_enter( e);
	}

	void dragMoveEvent( QDragMoveEvent *e) {
		emit on_drag_move( e);
	}

	void dragLeaveEvent( QDragLeaveEvent *e) {
		emit on_drag_leave( e);
	}

signals:
	void do_drag();
	void on_drop( QDropEvent *e);
	void on_drag_enter( QDragEnterEvent *e);
	void on_drag_move( QDragMoveEvent *e);
	void on_drag_leave( QDragLeaveEvent *e);

private:
	bool m_drag;
	bool m_drag_on_left;
	bool m_drag_on_middle;
	bool m_drag_on_right;
};
