/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "rg3DRom_physics.h"
#include "rg3DRom.h"

sk3DRomPhyBone::sk3DRomPhyBone()
{
	rag_doll = true;
}

bool sk3DRomPhyBone::sphere_collision( const sk3DRomPhyBone &b) const
{
	const rgVector &u = m_csys.map_point( sphPrisim.center);
	const rgVector &v = m_csys.map_point( b.sphPrisim.center);
	const float len = sphPrisim.radius + b.sphPrisim.radius;
	if( (u - v).length_squared() > (len * len)) return false;
	return true;
}

bool sk3DRomPhyBone::collision( const sk3DRomPhyBone &b) const
{
	if( !sphere_collision( b)) return false;
	return true;
}

// Description: Generates a sphere that contains the bone's "prisim-sphere" and
//	the "bone-sphere" of all slave bones.
rgPhyBoundSphere &sk3DRomPhyBone::gen_bone_sphere( const int bid,
	const sk3DRomPhySkel &skel)
{
	const std::vector <sk3DRomBoneLink> *link = skel.get_bone_links( bid);
	if( !link) return sphBone;
	std::vector <rgPhyBoundSphere> sph;
	sph.push_back( sphPrisim);
	for( int i = 0; i < int( link->size()); i++)
		sph.push_back( (skel.bone( ((*link)[i]).bid)).sphBone);
	return (sphBone.gen_containing( sph));
}

sk3DRomPhySkel::sk3DRomPhySkel( sk3DRomSkel &s, sk3DRomDrawContext &dc)
{
	//m_skel = new sk3DRomSkel( s);
	m_skel = &s;
	m_draw_context = &dc;
	//m_draw_context = new sk3DRomDrawContext();
}

sk3DRomPhySkel::~sk3DRomPhySkel()
{
	//if( m_skel) delete m_skel;
	//if( m_draw_context) delete m_draw_context;
}

int sk3DRomPhySkel::bone_count() const { return m_skel->bone_count();}

const std::vector <sk3DRomBoneLink> *sk3DRomPhySkel::get_bone_links(
	const int bid) const
{
	if( (bid < 0) || (bid >= m_skel->bone_count())) return 0;
	return &(m_skel->b[bid].link);
}

const std::vector <short> *sk3DRomPhySkel::get_master_list( const int bid)
	const
{
	if( (bid < 0) || (bid >= m_skel->bone_count())) return 0;
	return &(m_skel->b[bid].mlist);
}

/*
rgPhyBoundSphere sk3DRomPhySkel::gen_skel_sphere()
	const
{
	std::vector <float> span;
	{ for( int i = 0; i < int( m_skel->bone_count()); i++)
		span.push_back( bone_span( i));
	}
	if( span.empty()) return rgPhyBoundSphere();

	int msb = 0;	// Minimum spanning bone
	{
		float min = 16777816.0f;
		for( int i = 0; i < int( span.size()); i++) {
			if( span[i] < min) { msb = i; min = span[i];}
		}
	}
	return rgPhyBoundSphere( m_bone[ blist[msb]].m_csys.p, span[msb]);
}

float sk3DRomPhySkel::slave_span( int bid) const
{
	const std::vector <sk3DRomBoneLink> *link = get_bone_links( bid);
	if( !link) return 0;
	float span = m_bone[bid].local_span();
	for( int i = 0; i < int( link->size()); i++)
		span = RG_MAX( span, slave_span( (*link)[i].bid));
	return span;
}

float sk3DRomPhySkel::master_span( int bid) const
{
	const std::vector <short> *mst = get_master_list( bid);
	if( !mst) return 0;
	float span = m_bone[bid].local_span();
	for( int i = 0; i < int( mst->size()); i++)
		span = RG_MAX( span, master_span( (*mst)[i]));
	return span;
}
*/

void sk3DRomPhySkel::transform_phy_bone( const sk3DRomSkelFrame &s,
	const int bid)
{
	m_bone[bid].m_csys = s.b[bid].rcs;
}

