/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef RG3DROM_PHYSICS_H
#define RG3DROM_PHYSICS_H

#include <rgPhysics.h>

class sk3DRomSkel;
class sk3DRomPhySkel;
class sk3DRomBoneLink;
class sk3DRomDrawContext;
class sk3DRomSkelFrame;

class sk3DRomPhyBone
{
public:
	bool rag_doll;
	rgCoordSystem m_csys;

	sk3DRomPhyBone();

	// XXX: "m_prisim" must contain the transformed prisims for the
	//	desired skeleton frame.
	void gen_prisim_sphere() { sphPrisim.gen_containing( m_prisim);}

	rgPhyBoundSphere &gen_bone_sphere( const int bid,
		const sk3DRomPhySkel &skel);

	rgPhyBoundSphere &bone_sphere() { return sphBone;}

	float local_span() const {
		return RG_MAX( (sphPrisim.center - m_csys.p).length() +
			sphPrisim.radius, m_bone_length);
	}

	bool sphere_collision( const sk3DRomPhyBone &b) const;
	bool collision( const sk3DRomPhyBone &b) const;

protected:
	float m_bone_length;
	std::vector <rgCollisionPrisim> m_prisim;
	rgPhyBoundSphere sphBone;
	rgPhyBoundSphere sphPrisim;
};


class sk3DRomPhySkel : public rgPhyObject
{
public:
	sk3DRomPhySkel( sk3DRomSkel &s, sk3DRomDrawContext &dc);
	virtual ~sk3DRomPhySkel();

	const std::vector <sk3DRomBoneLink> *get_bone_links( const int bid)
		const;
	const std::vector <short> *get_master_list( const int bid) const;

	int bone_count() const;
	const sk3DRomPhyBone &bone( const int bid) const { return m_bone[bid];}

	std::vector <sk3DRomPhyBone> &bone_list() { return m_bone;}

	sk3DRomSkel *skel() { return m_skel;}

	// Description: Returns a sphere that contains all spheres in the
	//	skeleton. The sphere will be centered on the
	//	"minimum-spanning-bone" (this is the bone that produces the
	//	smallest sphere which still contains the entire skeleton).
	//	The skeleton containing sphere will contain the entire skeleton
	//	regardless of the orientation of the bones (except if bone
	//	joints are translated/disjointed).
	//
	// XXX: Must generate "prisim-spheres" for all bones in the skeleton
	//	before calling this function.
	//rgPhyBoundSphere gen_skel_sphere() const;

	void transform_phy_bone( const sk3DRomSkelFrame &s, const int bid);

protected:
	sk3DRomSkel *m_skel;
	sk3DRomDrawContext *m_draw_context;
	std::vector <sk3DRomPhyBone> m_bone;

	rgPhyBoundSphere sphSkeleton;

	// XXX: Must transform the physical skeleton at least once before
	//	calling any of the span functions.
	//float bone_span( int bid) const {
	//	return RG_MAX( slave_span( bid), master_span( bid));}
	//float master_span( int bid) const;
	//float slave_span( int bid) const;

private:
	sk3DRomPhySkel( const sk3DRomPhySkel &s);
	sk3DRomPhySkel &operator =( const sk3DRomPhySkel &s);
};

#endif
