/*
Copyright (C) 1997-2005 Amour Hassan
                                                                                
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
                                                                                
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                                                                                
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                                
*/

#ifndef RGBMP_H
#define RGBMP_H

#include <fstream>
#include <rgStandard.h>

#define RGBMP_FORCE_RGB 0x00000001
#define RGBMP_FORCE_RGBA 0x00000002	/* NOTE: RGBMP_FORCE_RGBA always
					 * 	implies RGBMP_FORCE_RGB.
					 */

/* Flags for rgBMP::convertXtoY() functions */
#define RGBMP_16BIT_1555	2000
#define RGBMP_16BIT_5551	2001
#define RGBMP_16BIT_565		2002


#define BMP_DATABEGIN_OFF 10
#define BMP_WIDTH_OFF 18
#define BMP_HEIGHT_OFF 22
#define BMP_BITCOUNT_OFF 28
#define BMP_PALETTE_OFF 54


struct rgBMPPaletteEntry
{
	rgBYTE red;
	rgBYTE green;
	rgBYTE blue;
	rgBYTE alpha;	/* Not normally used. */
};

class rgBMP;

class rgBumpMap
{
public:
	int width;
	int height;
	rgFloat *data;
	rgVector *nmap; /* Normal map */

	rgBumpMap();
	~rgBumpMap();

	void clear();
	void invert();
	int load( const char *fname);
	void genNormalMap();
	void scaleData( const rgFloat k);
	void scaleNormals( const rgFloat x, const rgFloat y, const rgFloat z);
	void normalize();

	int gen_texture_bump_map( rgBMP &bmp) const;
};

class rgBMP
{
public:

	rgBMP();
	~rgBMP();

	/* Resets the class to the state it had when it was first constructed.
	 */
	void clear();

	void convertBGRtoRGB();
	void convertBGRtoRGB( rgBYTE *p_imgData, const int p_pixelCount);

	/* Retrieving BMP information */
	int getColorDepth() const { return colorDepth;}
	int getHeight() const { return height;}
	rgBYTE *getImageData() { return imageData;}
	rgBMPPaletteEntry *getPalette() { return palette;}
	int getWidth() const { return width;}

	void get_sub_image( rgBYTE *p_bmpData, int x1, int y1, int x2, int y2,
		int row_stride);

	int copy_sub_image( rgBMP &src, int srcX, int srcY, int p_width,
		int p_height, int desX, int desY);

	/* Description: Extracts the width and height of a bitmap image
	 *	without actualy loading the entire image. This is useful when
	 *	only the dimensions of the image are required.
	 */
	int query_width_and_height( const char *fname, int &p_width,
		int &p_height);

	/* Loading a BMP file */
	int load( const char *fname);
	int load( const char *fname, int flags);
	int load( const char *fname, int flags, int maxWidth, int maxHeight);

	/* Saves the BMP currently loaded into rgBMP to a given file. */
	int saveThis( const char *fname) const {
		// FIXME: Saving the palette is not supported but should be.
		return save( fname, getWidth(), getHeight(), getColorDepth(),
			imageData, /*palette*/ 0);
	}

	/* Saves a given BMP to a file. */
	// FIXME: Saving the palette is not supported but should be.
	int save( const char *fname, const int width, const int height,
		const int p_colour_depth, const rgBYTE *p_bmpData,
		const rgBYTE *p_palette) const;

	void flip_vertically();

	int generate_bmp( const int w, const int h, const int p_depth);

protected:

	int width;
	int height;
	short colorDepth;

	rgBYTE *imageData;
	rgBMPPaletteEntry *palette;

	/* Normal bmp loading */
	int read8to8( std::ifstream &bmp);
	int read16to16( std::ifstream &bmp);
	int read24to24( std::ifstream &bmp);

	/* Loading bmp's and converting to 24 bit regardless of original
	 * colorDepth from the file.
	 */
	int read8to24( std::ifstream &bmp);
	int read16to24( std::ifstream &bmp);

	int write8( std::ofstream &bmp);
	int write16( std::ofstream &bmp);
	int write24( std::ofstream &bmp, const int p_width, const int p_height,
		const rgBYTE *p_imgData);


	int convert8to24( const int p_width, const int p_height,
		rgBYTE *img24, rgBYTE *img8,
		rgBYTE *p_palette);
	
	int convert16to24( const int p_width, const int p_height,
		rgBYTE *img24, rgWORD *img16, int p_format);

	int convert24to32( const int p_width, const int p_height,
		rgBYTE *img32, rgBYTE *img24);
};

#endif
