/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef RGBSP_GRR_H
#define RGBSP_GRR_H

#include <rgBSP.h>

class rgBSPFaceGRR
{
public:
	int count;
	int *v;
	rgVector normal;

	rgBSPFaceGRR;
};


class rgBSPNodeGRR
{
public:
	rgVector plane_point;
	rgVector normal;

	// List of faces beloning in this node.
	// NOTE: Faces are only stored in "leaf" nodes. That is, if "f" is
	//	not empty (contains faces), then both "front" and "back" will
	//	be NULL.
	std::vector <rgBSPFaceGRR> f;

	rgBSPNodeGRR *front;
	rgBSPNodeGRR *back;

	rgBSPNodeGRR();
};

template <class T>
class rgBSPTreeGRR
{
public:
	rgBSPNodeGRR *root;

	// BSP face rendering function.
	void (*chrf)( T *v, const int count);
	void (*interp)( T *des, T *srcA, T *srcB, const float k);

	rgBSPTreeGRR() {
		root = 0;
		chrf = 0;
	}

	~rgBSPTreeGRR() {
		clear();
	}

	void clear() {
		chrf = 0;
		interp = 0;
		
		if( root) { delete [] root; root = 0;}
		if( fdata) { delete [] fdata; fdata = 0;}
		if( fviddata) { delete [] fviddata; fviddata = 0;}
		if( vdata) { delete [] vdata; vdata = 0;}

		fcount = 0;
		fvidcount = 0;
		vcount = 0;
	}

	// Description: Builds the GRR BSP tree and makes it reay for use.
	//	"gen_GRR_BSP" handles all of the memory
	//	allocations so non of this class's memory should be manually
	//	allocated or deallocated.
	int gen_GRR_BSP( rgBSPTree &tree, void (*p_interp)( T*, T*, T*,
		const float)) {

		clear();

		if( !p_interp) return -1;
		interp = p_interp;

		// Determine the total number of verticies contained in the
		// BSP tree.

		return 0;
	}

	void render( rgVector p);
	void render( rgVector p, rgBSPNode *node);

private:
	// Array holding all of the faces used in the BSP tree.
	int fcount;
	int *fdata;

	// Array holding the vertex indicies used by all faces in the BSP
	// tree.
	int fvidcount;
	int *fviddata;

	// Array holding all of the verticies used in the BSP tree.
	int vcount;
	T *vdata;
}

#endif
