/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "rgNetwork.h"
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <netinet/in.h>
#include <netdb.h>

rgSocket::rgSocket()
{
	m_good = false;
	m_port = 0;
	m_backlog = 10;

	m_sockfd = 0;
	m_remotefd = 0;

	memset( &m_local_addr, 0, sizeof( m_local_addr));
	memset( &m_remote_addr, 0, sizeof( m_remote_addr));
}

/* XXX: Copying of rgSockets is not supported.
rgSocket::rgSocket( const rgSocket &s)
{
	m_good = false;
	m_sockfd = 0;
	m_remotefd = 0;
	do_copy( s);
}
*/

rgSocket::~rgSocket()
{
	close();
}

int rgSocket::open_listen_port( const int p_port, const int p_type)
{
	close();
	if( p_type <= 0) return -1000;
	m_type = p_type;
	m_port = p_port;
	if( -1 == (m_sockfd = socket( AF_INET, m_type, 0))) {
		m_sockfd = 0;
		return -1;
	}
	m_optval = 1;	// FIXME: What does this mean?
	if( -1 == setsockopt( m_sockfd, SOL_SOCKET, SO_REUSEADDR, &m_optval,
		sizeof( m_optval))) return -2;

	m_local_addr.sin_family = AF_INET;
	m_local_addr.sin_port = htons( port());
	m_local_addr.sin_addr.s_addr = INADDR_ANY;
	memset( &(m_local_addr.sin_zero), '\0', 8);

	if( -1 == bind( m_sockfd, (struct sockaddr *)&m_local_addr,
		sizeof(struct sockaddr))) return -3;

	if( listen( m_sockfd, m_backlog) == -1) return -4;

	m_good = true;
	return 0;
}

int rgSocket::accept()
{
	if( !is_good()) return -1;
	if( !m_sockfd) return -2;
	close_remote();
	int sin_size = sizeof( sockaddr_in);
	m_remotefd = rgsocket_accept( m_sockfd, (sockaddr *)&m_remote_addr,
		(socklen_t *)&sin_size);
	if( -1 == m_remotefd) {
		m_remotefd = 0;
		return -3;
	}
	return 0;
}

int rgSocket::connect( const char *ip, const int p_port, const int p_type)
{
	close();
	if( !ip) return -1;
	if( p_type <= 0) return -1000;
	m_type = p_type;
	m_port = p_port;

	struct hostent *he = gethostbyname( ip);
	if( NULL == he) return -2;

	m_remote_addr.sin_family = AF_INET;
	m_remote_addr.sin_port = htons( port());
	m_remote_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset( &(m_remote_addr.sin_zero), '\0', 8);

	if( -1 == (m_remotefd = socket( AF_INET, m_type, 0))) {
		m_remotefd = 0;
		return -3;
	}
	
	if( -1 == rgsocket_connect( m_remotefd, (sockaddr *)&m_remote_addr,
		sizeof(struct sockaddr))) return -4;

	m_good = true;
	return 0;
}

