/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
 * rgSDK/rgRay/rgRay.h
 *
 * Copyright 2004 Amour Hassan
 *
 */

#ifndef RG_RAY_H
#define RG_RAY_H

#include "rgStandard.h"
#include "rgPort.h"

#define RGRAY_NO_OBJECT 0
#define RGRAY_SPHERE 1
#define RGRAY_TRIANGLE 2

#define RGRAY_MAX_REFLECTIONS 10
#define RGRAY_MAX_TRACE_RECURSE 10

class rgRayReflec
{
public:
	rgFloat intensity;
	rgVector colour;
};

class rgRayColourMaterial
{
public:
	rgVector amb;
	rgVector dif;
	rgVector spec;
	rgFloat specExp;
};

class rgRayEntity
{
public:
	/* Unique identifier */
	rgLONG uid;
	rgRayEntity();
};

class rgRayLight : public rgRayEntity
{
public:
	rgVector position;

	rgVector colour;

	rgVector amb;
	rgVector dif;
	rgVector spec;
};

class rgRayObject : public rgRayEntity
{
public:
	rgVector colour;
	rgFloat reflec;
	rgFloat refrac; /* Index of refraction */
	rgFloat transparent;
	rgFloat specExp;

	rgVector amb;
	rgVector dif;
	rgVector spec;
};

class rgRaySphere : public rgRayObject
{
public:
	rgVector center;
	rgFloat radius;
};

class rgRayTriangle : public rgRayObject
{
public:
	rgVector a, b, c, n;
	void genNormal();
};

class rgRayFace : public rgRayObject
{
public:
	int vertCount;
	rgVector *vert;
};

class rgRayScene
{
public:
	char *outFileName;

	int viewWidth;
	int viewHeight;
	int traceDepth;
	rgFloat fieldOfView;

	rgVector backColour;

	int lightCount;
	int sphereCount;
	int triCount;
	int faceCount;

	rgRayLight *light;
	rgRaySphere *sphere;	
	rgRayTriangle *tri;
	rgRayFace *face;

	rgLONG getLightID( rgRayLight *p);
	rgLONG getSphereID( rgRaySphere *p);
	rgLONG getFaceID( rgRayFace *p);

	rgRayScene();	
};

class rgRayFrameBuffer
{
public:
	int width;
	int height;
	rgBYTE *colour;

	rgRayFrameBuffer();

	int create( const int p_width, const int p_height);
};

class rgRayCamera
{
public:

	rgVector position;

	/* Rotation around X, Y and Z axis */
	rgVector rotation;

	rgFloat fieldOfView;
	rgFloat viewPlaneDistance;

	int viewXResolution;
	int viewYResolution;

	rgVector getRayForPixel( const int p_x, const int p_y);
};


void rgRayTraceScene(
        rgFloat viewAngle,
        rgFloat aspect,
        rgFloat pnear,
        rgFloat pfar,
        rgRayFrameBuffer &frame,
	rgRayScene &scene);

bool rgRay_trace( rgVector rp, rgVector rd, rgRayScene &scene,
        rgVector &colour, rgLONG p_uid_to_skip, int depth);

rgFloat getSpecularScalar( rgVector liPos, rgVector fragPos, rgVector fragNorm,
	rgVector eyePos, rgFloat specExp);

rgFloat getDiffuseScalar( rgVector liPos, rgVector fragPos, rgVector fragNorm);

bool rgRayInShadow( rgRayScene &scene, rgVector rp, rgVector rd,
	rgLONG p_uid_to_skip);

void rgRayFindClosestSphere( rgRayScene &scene, rgVector rp, rgVector rd,
        int &sid, rgVector &ip, rgLONG p_uid_to_skip);

void rgRayFindClosestTriangle( rgRayScene &scene, rgVector rp, rgVector rd,
        int &tid, rgVector &ip, rgLONG p_uid_to_skip);

/*
 * Description: Computes the colour of a fragment due to direct illumination
 *	from lights in the scene. This function can also be used to determine
 *	if a fragment is in shadow. If rgRayLightFragment returns 0, then the
 *	fragment is in shadow. If the fragment was not totally in shadow, then
 *	the number of lights that illuminate the fragment is returned. 
 *
 * Return: 0 is returned if the given fragment is totaly in shadow (was not
 *	lit by any lights). Otherwise, the number of lights that directly
 *	light the fragment is returned. Also, if the fragment was light by
 *	at least one light, then the parameter "colour" will contain the final
 *	fragment colour due to the lights. The parameter "colour" will be
 *	black (0.0f, 0.0f, 0.0f) if the fragment was not lit by any lights.
 */
int rgRayLightFragment( rgVector fragPos, rgRayColourMaterial fragMat,
	rgVector fragNorm,
	rgVector &colour, rgRayScene &scene, rgLONG p_uid_to_skip);

#endif
