/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <rgOpenGL.h>
#include <rg3DRom.h>
#include "engine/engine_game.h"
#include "engine/engine_graphics.h"
#include "player.h"
#include "arena.h"
#include "team.h"

extern "C" gmThing *spw_phPerson()
{
	return new phPerson();
}

extern "C" gmThing *spw_phPlayer()
{
	return new phPlayer();
}

extern "C" int del_phPerson( gmThing *t)
{
	delete t;
	return 0;
}

extern "C" int del_phPlayer( gmThing *t)
{
	delete t;
	return 0;
}

RPH_THING_MODULE_MAIN_FUNCTION( phPerson)
{
	p_ge->m_thing_manager->add_to_spawn_list( "phPerson", spw_phPerson);
	p_ge->m_thing_manager->add_to_spawn_list( "phPlayer", spw_phPlayer);

	p_ge->m_thing_manager->add_to_delete_list( "phPerson", del_phPerson);
	p_ge->m_thing_manager->add_to_delete_list( "phPlayer", del_phPlayer);
	
	return 0;
}

phPerson::phPerson()
{
	m_arena = 0;
	m_team = 0;
}

phPerson::~phPerson()
{
}

phTeam *phPerson::team()
{
	if( (!m_team) || m_team->is_free()) return 0;
	// FIXME: This is a safety check which is probably not neccasary.
	if( m_team->m_thing->is_a( "phTeam"))
		return (phTeam *)(m_team->m_thing);
	else return 0;
}

phArena *phPerson::arena()
{
	if( !m_arena) return 0;
	// FIXME: This is a safety check which is probably not neccasary.
	if( m_arena->m_thing->is_a( "phArena"))
		return (phArena *)(m_arena->m_thing);
	else return 0;
}


phPlayer::phPlayer()
{
}

phPlayer::~phPlayer()
{
}

//
// XXX NOTES: When shadows are enabled, ONLY the player's shadow is rendered,
//	nothing else.
void phPlayer::draw( gmRenderArgs *p_ra)
{
	if( !p_ra) return;
	gmRenderArgs ra( *p_ra);

	// XXX: The arena already disabled transformation for transparency.
	// if( ra.render_transparent) ra.transform = false;
	if( ra.render_shadow) {

		ModelHandle *mh = ((ge())->gfx_engine->model_man())->get(
			"mdl_player_shadow");
		if( mh && (mh->model)) {

			(mh->model)->transform_skel_frame( m_draw_context);
			(mh->model)->render_vertex_array( 0);
		}
		return;
	}

	// Render the full opaque part of the model OR render the
	// transparent parts of the model. Transformation for the model
	// is suppressed if transparent parts will be rendered.
	if( ra.render_reflections) ra.lock_vertices = true;
	gmModelThing::draw( &ra);

	if( ra.render_reflections && !(ra.render_transparent)) {

		glPushMatrix();
		glScalef( 1, -1, 1);
		glCullFace( GL_FRONT);

		ra.transform = false;
		gmModelThing::draw( &ra);

		glCullFace( GL_BACK);
		glPopMatrix();
	}
}

