/*
Copyright (C) 2004-2005 Amour Hassan
                                                                                
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
                                                                                
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                                                                                
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                                
*/

#include "things.h"
#include "engine/engine_game.h"

extern "C" gmThing *spw_model_thing();
extern "C" gmThing *spw_physical_model();

extern "C" int del_model_thing( gmThing *t);
extern "C" int del_physical_model( gmThing *t);

//////////////////////////////////////////////////////////////////////////////
// XXX: DEV_SPAWNABLE_THINGS
//	Spawnable things are added here. If you want a thing to be spawnable,
//	then add it to the spawn list.
void ThingManager::init_spawn_list()
{
	add_to_spawn_list( "gmModelThing", spw_model_thing);
	add_to_spawn_list( "phyModel", spw_physical_model);

	add_to_delete_list( "gmModelThing", del_model_thing);
	add_to_delete_list( "gmPhyModel", del_physical_model);
}
//////////////////////////////////////////////////////////////////////////////

int ThingManager::add_to_spawn_list( const std::string &key, gmThing *(*fn)())
{
	typedef std::pair <const std::string, gmThing *(*)()> STPair;
	if( false == ((m_spawn.insert( STPair( key, fn)).second)))
		return TM_ERR_SAME_KEY;
	return 0;
}

int ThingManager::add_to_delete_list( const std::string &key,
	int (*fn)( gmThing *))
{
	typedef std::pair <const std::string, int (*)( gmThing *)> STPair;
	if( false == ((m_delete.insert( STPair( key, fn)).second)))
		return TM_ERR_SAME_KEY;
	return 0;
}

int ThingManager::spawn( const std::string &key, const std::string &type)
{
	std::map <const std::string, gmThing *(*)()> :: const_iterator iter =
                m_spawn.find( type);
	if( iter == m_spawn.end()) return TM_ERR_NO_THING;
	return create_thing( key, (iter->second)());
}

std::string ThingManager::list_spawnable() const
{
	std::string tmp( "Spawnable Things:\n");
	std::map <const std::string, gmThing *(*)()> :: const_iterator iter =
                m_spawn.begin();
	for( ; iter != m_spawn.end(); iter++)
		tmp += "\t\"" + (iter->first) + "\"\n";
	tmp += "Total Spawnable: " + gm_to_string( int( m_spawn.size())) +
		"\n";
	return tmp;
}


